@@ -21,10 +21,12 @@ from typing import Optional
 import collections
 import heapq
 
+import numpy as np
+
 from slidingpuzzle.board import (
     Board,
     FrozenBoard,
-    get_empty_yx,
+    find_blank,
     get_next_moves,
     is_solvable,
     shuffle_board,
@@ -60,16 +62,16 @@ def get_next_states(state: State) -> list[State]:
     """
     Creates a list of next viable states, given the current state.
     """
-    moves = get_next_moves(state.board, state.empty_pos)
+    moves = get_next_moves(state.board, state.blank_pos)
     next_states = []
     for move in moves:
-        # construct new altered board
-        next_board = tuple(row.copy() for row in state.board)
-        swap_tiles(next_board, state.empty_pos, move)
-        # record history
+        # construct altered board
+        next_board = np.copy(state.board)
+        swap_tiles(next_board, state.blank_pos, move)
+        # update history
         next_history = state.history.copy()
         next_history.append(move)
-        # after moving, the move_pos is now the empty_pos
+        # after moving, the move is now the blank_pos
         next_state = State(next_board, move, next_history)
         next_states.append(next_state)
     return next_states
@@ -111,9 +113,8 @@ def a_star(board: Board, **kwargs) -> SearchResult:
     weight = kwargs.get("weight", 1)
 
     # initial state
-    goal = new_board(len(board), len(board[0]))
-    empty_pos = get_empty_yx(board)
-    initial_state = State(board, empty_pos)
+    goal = new_board(*board.shape)
+    initial_state = State(np.copy(board), find_blank(board))
     unvisited = [initial_state]
     visited: set[FrozenBoard] = set()
 
@@ -125,7 +126,7 @@ def a_star(board: Board, **kwargs) -> SearchResult:
         expanded += 1
 
         # goal check
-        if goal == state.board:
+        if np.array_equal(goal, state.board):
             return SearchResult(
                 board, generated, expanded, unvisited, visited, state.history
             )
@@ -164,7 +165,7 @@ def beam(board: Board, **kwargs) -> SearchResult:
             Default is ``True``.
         heuristic: A function that maps boards to an estimated cost-to-go.
             Default is :func:`slidingpuzzle.heuristics.linear_conflict_distance`.
-        width (int): The beam width. Default is ``3``.
+        width (int): The beam width. Default is ``4``.
 
     Returns:
         A :class:`slidingpuzzle.state.SearchResult` with a solution and statistics
@@ -174,44 +175,47 @@ def beam(board: Board, **kwargs) -> SearchResult:
     f_bound = kwargs.get("f_bound", float("inf"))
     detect_dupes = kwargs.get("detect_dupes", True)
     heuristic = kwargs.get("heuristic", linear_conflict_distance)
-    width = kwargs.get("width", 3)
+    width = kwargs.get("width", 4)
 
     # initial state
-    goal = new_board(len(board), len(board[0]))
-    empty_pos = get_empty_yx(board)
-    initial_state = State(board, empty_pos)
-    unvisited = collections.deque([initial_state])
+    goal = new_board(*board.shape)
+    initial_state = State(np.copy(board), find_blank(board))
+    unvisited = [initial_state]
     visited: set[FrozenBoard] = set()
 
     # stats
     generated, expanded = 0, 0
 
     while unvisited:
-        state = unvisited.popleft()
-        expanded += 1
+        next_level = []
+        while unvisited:
+            state = unvisited.pop()
+            expanded += 1
 
-        # goal check
-        if goal == state.board:
-            return SearchResult(
-                board, generated, expanded, unvisited, visited, state.history
-            )
+            # goal check
+            if np.array_equal(goal, state.board):
+                return SearchResult(
+                    board, generated, expanded, unvisited, visited, state.history
+                )
 
-        # bound
-        if len(state.history) > depth_bound or state.f > f_bound:
-            continue
+            # bound
+            if len(state.history) > depth_bound or state.f > f_bound:
+                continue
 
-        # duplicate detection
-        if detect_dupes and visit(visited, state.board):
-            continue
+            # duplicate detection
+            if detect_dupes and visit(visited, state.board):
+                continue
 
-        # children
-        next_states = get_next_states(state)
-        for state in next_states:
-            state.f = heuristic(state.board)
-        next_states.sort()
-        next_states = next_states[:width]
-        unvisited.extend(next_states)
-        generated += len(next_states)
+            # children
+            next_states = get_next_states(state)
+            for state in next_states:
+                state.f = heuristic(state.board)
+            next_level.extend(next_states)
+            generated += len(next_states)
+
+        # sort all children at this level
+        next_level.sort(reverse=True)
+        unvisited = next_level[-width:]
 
     # if we are here, no solution was found
     return SearchResult(board, generated, expanded, list(unvisited), visited, None)
@@ -235,9 +239,8 @@ def bfs(board: Board, **kwargs) -> SearchResult:
     detect_dupes = kwargs.get("detect_dupes", True)
 
     # initial state
-    goal = new_board(len(board), len(board[0]))
-    empty_pos = get_empty_yx(board)
-    initial_state = State(board, empty_pos)
+    goal = new_board(*board.shape)
+    initial_state = State(np.copy(board), find_blank(board))
     unvisited = collections.deque([initial_state])
     visited: set[FrozenBoard] = set()
 
@@ -249,7 +252,7 @@ def bfs(board: Board, **kwargs) -> SearchResult:
         expanded += 1
 
         # goal check
-        if goal == state.board:
+        if np.array_equal(goal, state.board):
             return SearchResult(
                 board, generated, expanded, unvisited, visited, state.history
             )
@@ -289,9 +292,8 @@ def dfs(board: Board, **kwargs) -> SearchResult:
     detect_dupes = kwargs.get("detect_dupes", True)
 
     # initial state
-    goal = new_board(len(board), len(board[0]))
-    empty_pos = get_empty_yx(board)
-    initial_state = State(board, empty_pos)
+    goal = new_board(*board.shape)
+    initial_state = State(np.copy(board), find_blank(board))
     unvisited = [initial_state]
     visited: set[FrozenBoard] = set()
 
@@ -303,7 +305,7 @@ def dfs(board: Board, **kwargs) -> SearchResult:
         expanded += 1
 
         # goal check
-        if goal == state.board:
+        if np.array_equal(goal, state.board):
             return SearchResult(
                 board, generated, expanded, unvisited, visited, state.history
             )
@@ -349,9 +351,8 @@ def greedy(board: Board, **kwargs) -> SearchResult:
     heuristic = kwargs.get("heuristic", linear_conflict_distance)
 
     # initial state
-    goal = new_board(len(board), len(board[0]))
-    empty_pos = get_empty_yx(board)
-    initial_state = State(board, empty_pos)
+    goal = new_board(*board.shape)
+    initial_state = State(np.copy(board), find_blank(board))
     unvisited = [initial_state]
     visited: set[FrozenBoard] = set()
 
@@ -363,7 +364,7 @@ def greedy(board: Board, **kwargs) -> SearchResult:
         expanded += 1
 
         # goal check
-        if goal == state.board:
+        if np.array_equal(goal, state.board):
             return SearchResult(
                 board, generated, expanded, unvisited, visited, state.history
             )
@@ -395,7 +396,8 @@ def ida_star(board: Board, **kwargs) -> SearchResult:
 
     Args:
         board: The board
-        depth_bound (int): A limit to search depth. Default is :math:`\infty`.
+        depth_bound (int): A limit to search depth. Default is :math:`\infty`. This is
+            an optional bound used in addition to the default f-bound.
         detect_dupes (bool): Duplicate detection (i.e. track visited states).
             Default is ``True``.
         heuristic: A function that maps boards to an estimated cost-to-go.
@@ -413,10 +415,9 @@ def ida_star(board: Board, **kwargs) -> SearchResult:
     weight = kwargs.get("weight", 1)
 
     # initial state
-    goal = new_board(len(board), len(board[0]))
-    empty_pos = get_empty_yx(board)
+    goal = new_board(*board.shape)
+    initial_state = State(np.copy(board), find_blank(board))
     bound = float(heuristic(board))
-    initial_state = State(board, empty_pos)
     next_bound = float("inf")
     unvisited = [initial_state]
     visited: set[FrozenBoard] = set()
@@ -430,7 +431,7 @@ def ida_star(board: Board, **kwargs) -> SearchResult:
             expanded += 1
 
             # goal check
-            if goal == state.board:
+            if np.array_equal(goal, state.board):
                 return SearchResult(
                     board, generated, expanded, unvisited, visited, state.history
                 )
@@ -467,11 +468,13 @@ def ida_star(board: Board, **kwargs) -> SearchResult:
 
 def iddfs(board: Board, **kwargs) -> SearchResult:
     r"""
-    Iterative deepening depth first search. Same as :func:`dfs`, except that the depth
-    bound is incrementally increased until a solution is found.
+    Iterative deepening depth first search. Similar to :func:`dfs`, except that the
+    depth bound is incrementally increased until a solution is found.
 
     Args:
         board: The board
+        depth_bound (int): The initial bound. Default is
+            :func:`slidingpuzzle.heuristics.linear_conflict_distance`.
         detect_dupes (bool): Duplicate detection (i.e. track visited states).
             Default is ``True``.
 
@@ -479,14 +482,13 @@ def iddfs(board: Board, **kwargs) -> SearchResult:
         A :class:`slidingpuzzle.state.SearchResult` with a solution and statistics
     """
     # args
-    detect_dupes = kwargs.get("detect_dupes", True)
+    depth_bound = kwargs.get("depth_bound", linear_conflict_distance(board))
+    detect_dupes = kwargs.get("detect_dupes", False)
 
     # initial state
-    bound = linear_conflict_distance(board)
-    goal = new_board(len(board), len(board[0]))
-    empty_pos = get_empty_yx(board)
-    initial_state = State(board, empty_pos)
-    next_bound = bound
+    goal = new_board(*board.shape)
+    initial_state = State(np.copy(board), find_blank(board))
+    next_bound = depth_bound
     unvisited = [initial_state]
     visited: set[FrozenBoard] = set()
 
@@ -499,13 +501,13 @@ def iddfs(board: Board, **kwargs) -> SearchResult:
             expanded += 1
 
             # goal check
-            if goal == state.board:
+            if np.array_equal(goal, state.board):
                 return SearchResult(
                     board, generated, expanded, unvisited, visited, state.history
                 )
 
             # bound
-            if len(state.history) > bound:
+            if len(state.history) > depth_bound:
                 next_bound = len(state.history)
                 continue
 
@@ -519,8 +521,8 @@ def iddfs(board: Board, **kwargs) -> SearchResult:
             generated += len(next_states)
 
         # do we need to restart?
-        if bound != next_bound:
-            bound = next_bound
+        if depth_bound != next_bound:
+            depth_bound = next_bound
             unvisited.append(initial_state)
             visited.clear()
 
@@ -567,7 +569,7 @@ def search(board: Board, alg: str = A_STAR, **kwargs) -> SearchResult:
 
     Args:
         board: The initial board state to search.
-        algorithm (str): The algorithm to use for search.
+        alg: The algorithm to use for search.
             Use ``print(ALGORITHMS)`` to see available.
         kwargs: Algorithm arguments as described above.
 
@@ -588,7 +590,7 @@ def evaluate(
     h: int,
     w: int,
     heuristic=linear_conflict_distance,
-    algorithm=A_STAR,
+    alg=A_STAR,
     num_iters: int = 64,
     **kwargs,
 ) -> float:
@@ -600,7 +602,7 @@ def evaluate(
         h: Height of the board
         w: Width of the board
         heuristic: Heuristic function to evaluate
-        algorithm: Search algorithm to evaluate
+        alg: Search algorithm to evaluate
         num_iters: Number of iterations to average
         kwargs: Additional args for ``algorithm``
 
@@ -611,7 +613,7 @@ def evaluate(
     for _ in range(num_iters):
         board = new_board(h, w)
         shuffle_board(board)
-        result = search(board, alg=algorithm, heuristic=heuristic, **kwargs)
+        result = search(board, alg=alg, heuristic=heuristic, **kwargs)
         total += result.generated
     return round(total / num_iters, 2)
 
