@@ -12,80 +12,75 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import random
-
+import numpy as np
 import pytest
 
 from slidingpuzzle import *
 
 
-def test_newboard():
-    ground_truth = ([1, 2, 3], [4, 5, 6], [7, 8, 0])
-    assert new_board(3, 3) == ground_truth
+def test_new_board():
+    ground_truth = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 0]])
+    assert np.array_equal(new_board(3, 3), ground_truth)
 
-    ground_truth = ([1, 2], [3, 4], [5, 0])
-    assert new_board(3, 2) == ground_truth
+    ground_truth = np.array([[1, 2], [3, 4], [5, 0]])
+    assert np.array_equal(new_board(3, 2), ground_truth)
 
 
-def test_freezeboard():
+def test_board_from():
+    assert np.array_equal(board_from([1, 2], [3, 0]), new_board(2, 2))
+
+
+@pytest.mark.parametrize("h", [3, 5])
+@pytest.mark.parametrize("w", [3, 5])
+def test_board_from_iter(h, w):
+    values = list(range(1, 1 + h * w))
+    values[-1] = BLANK_TILE
+    b = board_from_iter(h, w, values)
+    assert np.array_equal(b, new_board(h, w))
+
+
+def test_freeze_board():
     b = new_board(3, 3)
     frozen = freeze_board(b)
     assert id(b) != id(frozen)
     swap_tiles(b, (0, 1), (1, 0))
-    assert b[0][1] != frozen[0][1]
+    assert b[0, 1] != frozen[0][1]
 
 
 def test_print_board():
     print_board(new_board(5, 5))
 
 
-def test_get_yx():
-    board = new_board(3, 3)
-    tile = 1
+def test_get_goal_yx():
+    h, w = 3, 3
+    board = new_board(h, w)
     for y, row in enumerate(board):
-        for x in range(len(row)):
-            if board[y][x] == EMPTY_TILE:
-                continue
-            assert tile == board[y][x]
-            tile += 1
+        for x, tile in enumerate(row):
+            assert (y, x) == get_goal_yx(h, w, tile)
 
 
-def test_get_empty_yx():
+def test_find_blank():
     board = new_board(3, 3)
-    assert get_empty_yx(board) == (2, 2)
-    assert get_yx(board, EMPTY_TILE) == get_empty_yx(board)
+    assert find_blank(board) == (2, 2)
+    assert find_tile(board, BLANK_TILE) == find_blank(board)
 
-    tmp = board[2][2]
-    board[2][2] = board[0][1]
-    board[0][1] = tmp
-    assert get_empty_yx(board) == (0, 1)
+    board[0, 1], board[2, 2] = board[2, 2], board[0, 1]
+    assert find_blank(board) == (0, 1)
 
 
 def test_swap_tiles():
     board = new_board(3, 3)
     pos1 = (2, 2)
     pos2 = (1, 1)
-    orig_pos1 = board[pos1[0]][pos1[1]]
-    orig_pos2 = board[pos2[0]][pos2[1]]
+    orig_pos1 = board[pos1[0], pos1[1]]
+    orig_pos2 = board[pos2[0], pos2[1]]
     swap_tiles(board, pos1, pos2)
-    assert orig_pos1 == board[pos2[0]][pos2[1]]
-    assert orig_pos2 == board[pos1[0]][pos1[1]]
-
-
-def test_apply_move():
-    board = new_board(5, 5)
-    pos1 = (2, 2)
-    pos2 = get_empty_yx(board)
-    orig_pos1 = board[pos1[0]][pos1[1]]
-    orig_pos2 = board[pos2[0]][pos2[1]]
-    apply_move(board, pos1)
-    assert orig_pos1 == board[pos2[0]][pos2[1]]
-    assert orig_pos2 == board[pos1[0]][pos1[1]]
+    assert orig_pos1 == board[pos2[0], pos2[1]]
+    assert orig_pos2 == board[pos1[0], pos1[1]]
 
 
 @pytest.mark.parametrize("size", [(2, 2), (3, 2), (2, 3), (3, 3)])
 def test_shuffle_board(size):
-    random.seed(0)
     h, w = size
     board = new_board(h, w)
     shuffle_board(board)
@@ -95,7 +90,6 @@ def test_shuffle_board(size):
 
 @pytest.mark.parametrize("size", [(2, 2), (3, 2), (2, 3), (3, 3)])
 def test_shuffle_board_lazy(size):
-    random.seed(0)
     h, w = size
     board = new_board(h, w)
     shuffle_board_lazy(board)
@@ -120,3 +114,10 @@ def test_get_next_states():
     state = State(board, (2, 2), [])
     next_states = get_next_states(state)
     assert len(next_states) == 2
+
+
+@pytest.mark.parametrize("h", [3, 5])
+@pytest.mark.parametrize("w", [3, 5])
+def test_board_generator(h, w):
+    gen = board_generator(h, w)
+    assert np.array_equal(next(gen), board_from_iter(h, w, range(h * w)))
